﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;

namespace GameEditor
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
            //为了测试方便，只开了一个数据类型
            IList<Pair> lst = new List<Pair>();
            //lst.Add(new Pair() { Name = "整型(1Byte)", DataType = typeof(System.SByte) });
            lst.Add(new Pair() { Name = "整型(2Byte)", DataType = typeof(System.Int16) });
            //lst.Add(new Pair() { Name = "整型(4Byte)", DataType = typeof(System.Int32) });
            //lst.Add(new Pair() { Name = "整型(8Byte)", DataType = typeof(System.Int64) });
            //lst.Add(new Pair() { Name = "整型(1Byte,无符号)", DataType = typeof(System.Byte) });
            //lst.Add(new Pair() { Name = "整型(2Byte,无符号)", DataType = typeof(System.UInt16) });
            //lst.Add(new Pair() { Name = "整型(4Byte,无符号)", DataType = typeof(System.UInt32) });
            //lst.Add(new Pair() { Name = "整型(8Byte,无符号)", DataType = typeof(System.UInt64) });
            //lst.Add(new Pair() { Name = "Single", DataType = typeof(System.Single) });
            //lst.Add(new Pair() { Name = "Double", DataType = typeof(System.Double) });
            //lst.Add(new Pair() { Name = "String", DataType = typeof(System.String) });

            cmbDateType.DataSource = lst;
            cmbDateType.DisplayMember = "Name";
            cmbDateType.ValueMember = "DataType";
        }

        /// <summary>
        /// 查找按键的单击事件
        /// </summary>
        private void btnSearch_Click(object sender, EventArgs e)
        {
            List<Result> resultList = new List<Result>();
            String value = txtValue.Text.Trim();
            Type type = (Type)cmbDateType.SelectedValue;
            int size = 4 * 1024;//4KB，每次读取4K数据
            //从4MB开始读，0-4M是无法读取的，为了防止整数举出，都少乘了个4
            for (int i = 4 * 1024; i < 2 * 1024 * 1024; i += 4)
            {
                byte[] buffer = new byte[size];//从目标进程中读取的数据是保存在这个数组中
                int sizeOfRead;
                //这里的i要乘1024才是真正的地址， 这里调用Api读取内存
                bool isOk = MemoryUtils.ReadMemoryBlock(this.GetAttacthedProcessHandle(), (IntPtr)(i * 1024), buffer, size, out sizeOfRead);

                //对取出来的这4k内存与期待值进行匹配，返回这个数组中所有匹配的数据的起始偏移量
                IList<int> pos = MemoryUtils.SearchResult(buffer, MemoryUtils.GetBytes(value, type), type);

                if (pos != null && pos.Count > 0)
                {
                    foreach (int el in pos)
                    {
                        //i * 1024 + el这个是转化成真正的虚拟地址
                        resultList.Add(new Result((IntPtr)(i * 1024 + el), txtValue.Text.Trim(), cmbDateType.SelectedValue.ToString()));
                    }
                }

                if (!isOk)
                {
                    break;//如果读取失败，则退出，通常意味着读到最后了
                }
            }
            dgvResult.DataSource = resultList;
        }
        private IntPtr GetAttacthedProcessHandle()
        {
            return Process.GetProcessById(int.Parse(txtProcessId.Text.Trim())).Handle;
        }

        /// <summary>
        /// 在结果中查找的单击事件
        /// </summary>
        private void btnSearchInResult_Click(object sender, EventArgs e)
        {
            List<Result> resultList = (List<Result>)dgvResult.DataSource;
            List<Result> newResultList = new List<Result>();
            String value = txtValue.Text.Trim();
            Type type = (Type)cmbDateType.SelectedValue;

            byte[] inputBytes = MemoryUtils.GetBytes(value, type);
            int inputSize = inputBytes.Length;
            //对上一次查询出来的结果再次与新的期待值进行匹配
            foreach (Result result in resultList)
            {
                byte[] buffer = new byte[inputSize];
                int sizeOfRead = 0;
                //这里调用Api读取内存
                bool isOk = MemoryUtils.ReadMemoryBlock(this.GetAttacthedProcessHandle(), result.GetAddress(), buffer, inputSize, out sizeOfRead);
                if (isOk)
                {
                    bool isFit = true;
                    for (int i = 0; i < inputSize; i++)
                    {
                        isFit = isFit && (buffer[i] == inputBytes[i]);
                        if (!isFit)
                        {
                            break;
                        }
                    }
                    if (isFit)
                    {
                        newResultList.Add(new Result(result.GetAddress(), value, result.Type));
                    }
                }
            }

            dgvResult.DataSource = newResultList;

        }

        /// <summary>
        /// 结果列表的单元格双击事件
        /// </summary>
        private void dgvResult_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            //把双击的那一列加到任务列表中
            Result selectedResult = (Result)dgvResult.Rows[e.RowIndex].DataBoundItem;
            List<Task> taskList = (List<Task>)dgvTask.DataSource;
            if (selectedResult != null)
            {
                if (taskList == null)
                {
                    taskList = new List<Task>();
                }
                taskList.Add(new Task(selectedResult.GetAddress(), txtLockedValue.Text.Trim(), (Type)cmbDateType.SelectedValue));
                dgvTask.DataSource = taskList;
            }
        }

        /// <summary>
        /// Timer
        /// </summary>
        private void tmrWriteMem_Tick(object sender, EventArgs e)
        {
            //Timer对任务进行处理，每条任务都会往固定的内存写数据
            List<Task> taskList = (List<Task>)dgvTask.DataSource;
            if (taskList == null || taskList.Count == 0)
            {
                return;
            }
            foreach (Task task in taskList)
            {
                task.WriteMemory(this.GetAttacthedProcessHandle());
            }
        }


    }
}
